
import simplejson
import decimal
import traceback
import sqlalchemy
import datetime
import logging

from flask import request, render_template

from libraries.database.database import APP
from libraries.database.broadcasts import TrackService, Schedule

def alchemyencoder(obj):
    logger = logging.getLogger("AuraEngine")
    logger.warning(str(type(obj)))

    """JSON encoder function for SQLAlchemy special classes."""
    if isinstance(obj, datetime.date):
        return obj.isoformat()
    elif isinstance(obj, decimal.Decimal):
        return float(obj)
    elif isinstance(obj, sqlalchemy.orm.state.InstanceState):
        return ""
    elif isinstance(obj, Schedule):
        return simplejson.dumps([obj._asdict()], default=alchemyencoder)
    else:
        return str(obj)

class Routes:
    error = None
    scheduler = None
    messenger = None
    lqs_communicator = None

    def __init__(self, scheduler, lqs_communicator, messenger):
        self.logger = logging.getLogger("AuraEngine")

        self.scheduler = scheduler
        self.messenger = messenger
        self.lqs_communicator = lqs_communicator

        APP.run(debug=True)

    @staticmethod
    @APP.route('/')
    @APP.route('/index')
    def index():
        return ""
        return render_template("index.html")

    @staticmethod
    @APP.route("/trackservice", methods=["GET"])
    def track_service():
        from_time = request.args.get("from")
        to_time = request.args.get("to")
        last = request.args.get("last")
        now = request.args.get("now")

        # nothing set => today's playlist
        if from_time == None and to_time == None and now == None:
            selected_date = datetime.date.today()
            trackservice_entries = TrackService.select_by_day(selected_date)

        # from and end time set
        elif from_time is not None and to_time is not None:
            to_time = datetime.datetime.strptime(to_time, "%Y-%m-%d")
            from_time = datetime.datetime.strptime(from_time, "%Y-%m-%d")
            trackservice_entries = TrackService.select_by_range(from_time, to_time)

        # now set
        elif now == "":
            datetime.date.today()
            trackservice_entries = TrackService.now_playing()

        return render_template("trackservice.html",
                               length=len(trackservice_entries),
                               trackservice_entries=trackservice_entries,
                               selected_date=selected_date)

    @staticmethod
    @APP.route("/test")
    def test():
        return render_template("index2.html")

    @staticmethod
    @APP.route("/login")
    def login():
        return "login"
        #return render_template("index.html")

    @staticmethod
    @APP.route("/logout")
    def logout():
        session.pop("logged_in", None)
        return "logout"
        #return render_template("index.html")

    @staticmethod
    @APP.route("/api/v1/trackservice/<selected_date>", methods=["GET"])
    def api_trackservice(selected_date):
        tracks_on_selected_date = None

        # convert date
        try:
            selected_date = datetime.datetime.strptime(selected_date, "%Y-%m-%d").date()
            tracks_on_selected_date = TrackService.select_by_day(selected_date)

            tracks_on_selected_date_in_json = simplejson.dumps([tracks._asdict() for tracks in tracks_on_selected_date], default=alchemyencoder)
        except Exception as e:
            import traceback
            traceback.print_exc()

            logger = logging.getLogger("AuraEngine")
            logger.error("Cannot transform programme into JSON String. Reason: " + str(e))

            return simplejson.dumps({"Error": str(e)})

        return tracks_on_selected_date_in_json

    @staticmethod
    @APP.route("/api/v1/trackservice/", methods=["GET"])
    def api_trackservice_now():
        return simplejson.dumps({'reached': True})


